unit Main;

{$IFDEF FPC}
  {$MODE OBJFPC}
  {$LONGSTRINGS ON}
{$ENDIF}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, LCLType;

type
  TMainForm = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FormResize(Sender: TObject);
    procedure FormPaint(Sender: TObject);
  private
    VR, VI, VS: Double;
    procedure GenerateImage;
  public
    ImageBuffer: TBitmap;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.lfm}

procedure TMainForm.FormCreate(Sender: TObject);
begin
  ImageBuffer := TBitmap.Create;
  //ImageBuffer.PixelFormat := pf24bit;
  VR := -0.5;
  VI := 0;
  VS := 0.0025;
  FormResize(Self);
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  ImageBuffer.Free;
end;

procedure TMainForm.FormResize(Sender: TObject);
begin
  ImageBuffer.SetSize(ClientWidth, ClientHeight);
  GenerateImage;
end;

procedure TMainForm.FormPaint(Sender: TObject);
begin
  Canvas.Draw(0, 0, ImageBuffer);
end;

procedure TMainForm.FormMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  VR := VR+VS*(X-(ClientWidth div 2));
  VI := VI+VS*((ClientHeight div 2)-Y);
  if Button=mbRight then
    VS := VS*2
  else
    VS := VS/2;
  GenerateImage;
  Invalidate;
end;

procedure TMainForm.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  case Key of
  vk_Left: VR := VR-VS*(ClientWidth div 20);
  vk_Right: VR := VR+VS*(ClientWidth div 20);
  vk_Up: VI := VI+VS*(ClientHeight div 20);
  vk_Down: VI := VI-VS*(ClientHeight div 20);
  vk_Add: VS := VS/2;
  vk_Subtract: VS := VS*2;
  end;
  GenerateImage;
  RePaint;
end;

procedure TMainForm.GenerateImage;
var
  M, R, I: Double;
  ZR, ZI: Double;
  ZRL, ZIL: Double;
  K: Integer;
  StartX, MidX, EndX: Integer;
  StartY, MidY, EndY: Integer;
  X, Y: Integer;
  ColorValue: TColor;
begin
  ImageBuffer.BeginUpdate(True);
  StartX := 0;
  MidX := ClientWidth div 2;
  EndX := ClientWidth-1;
  StartY := 0;
  MidY := ClientHeight div 2;
  EndY := ClientHeight-1;
  for Y := StartY to EndY do
    for X := StartX to EndX do
      begin
        R := VR+Double(X-MidX)*VS;
        I := VI+Double(MidY-Y)*VS;
        M := R*R+I*I;
        if M<4 then
          begin
            ColorValue := clBlack;
            ZRL := 0;
            ZIL := 0;
            K := 0;
            repeat
              ZR := ZRL*ZRL-ZIL*ZIL+R;
              ZI := 2*ZRL*ZIL+I;
              ZRL := ZR;
              ZIL := ZI;
              M := ZR*ZR+ZI*ZI;
              if M>=4 then
                begin
                  ColorValue := RGBToColor(255-K, 255-K, 255-K);
                  Break;
                end;
              Inc(K);
            until K=256;
          end
        else
          ColorValue := clWhite;
        ImageBuffer.Canvas.Pixels[X, Y] := ColorValue;
      end;
  ImageBuffer.EndUpdate;
end;

(*
procedure TMainForm.GenerateImage;
var
  PrimaryColor: TColor;
  StartX, EndX: Integer;
  StartY, EndY: Integer;
  I, X, Y: Integer;
  PixelPosition: PByte;
begin
  ImageBuffer.BeginUpdate;
  for I := 0 to 2 do
    begin
      PrimaryColor := 255 shl (I * 8);
      StartY := 0;
      EndY := (ClientHeight div 2)-1;
      StartX := (ClientWidth div 3)*I;
      EndX := (ClientWidth div 3)*(I+1)-1;
      for Y := StartY to EndY do
        begin
          with ImageBuffer.RawImage do
            PixelPosition := Data+Description.BytesPerLine*Y+StartX*4;
          for X := StartX to EndX do
            begin
              case I of
              0:
                begin
                  PixelPosition^ := 255;
                  Inc(PixelPosition);
                  PixelPosition^ := 0;
                  Inc(PixelPosition);
                  PixelPosition^ := 0;
                end;
              1:
                begin
                  PixelPosition^ := 0;
                  Inc(PixelPosition);
                  PixelPosition^ := 255;
                  Inc(PixelPosition);
                  PixelPosition^ := 0;
                end;
              2:
                begin
                  PixelPosition^ := 0;
                  Inc(PixelPosition);
                  PixelPosition^ := 0;
                  Inc(PixelPosition);
                  PixelPosition^ := 255;
                end;
              end;
              Inc(PixelPosition);
              PixelPosition^ := 0;
              Inc(PixelPosition);
            end;
        end;
    end;
  StartY := (ClientHeight div 2);
  EndY := ClientHeight-1;
  StartX := 0;
  EndX := ClientWidth-1;
  for Y := StartY to EndY do
    begin
      with ImageBuffer.RawImage do
        PixelPosition := Data+Description.BytesPerLine*Y;
      for X := StartX to EndX do
        begin
          PixelPosition^ := Random(256);
          Inc(PixelPosition);
          PixelPosition^ := Random(256);
          Inc(PixelPosition);
          PixelPosition^ := Random(256);
          Inc(PixelPosition);
          PixelPosition^ := 0;
          Inc(PixelPosition);
        end;
    end;
  ImageBuffer.EndUpdate;
end;
*)
end.

